macroScript pb_TimeTagger category:"Piranha-Bytes" buttonText:"TimeTagger" toolTip:"TimeTag Editor"
(
	local ro_TagEditor, ro_TimeTaggerConfig, ro_CreateFromKeys, ro_TT_About

	local TE_TagPresets = #("None", "Selected Object Name")
	local TE_PATH
	local tag_DisplayList = #()
	local tag_List = #()
	
	local controllerList = #()
	local controllerNameList = #()
	local objectList = #()
	local objectNameList = #()
	
	local currentTags = #()
	local currentTags_Timer = #()
	
	local tagDisplayTime = 100
	
	function writeVPInformation textArray bgColor =
	(
		gw.resetUpdateRect()
		gw.setTransform(Matrix3 1)
		
		xWin = gw.getWinSizeX()
		yWin = gw.getWinSizeY()
		
		sPos = [xWin/2,yWin/2,0]
		
		LineCount = textArray.count
		tSizeArray = #()
		for tl = 1 to LineCount do append tSizeArray (gw.GetTextExtent textArray[tl]).x
		maxX = amax tSizeArray
		ySize1 = (gw.GetTextExtent textArray[1]).y
		ySize = ySize1 * LineCount	
		spos.x = sPos.x - (maxX / 2)
		spos.y = sPos.y - 80
		gw.setColor #fill bgColor
		gw.setColor #line bgColor

		for d = (- (ySize / 2)) to 4 do gw.hPolyline #(spos - [d,d, 0], [spos.x + maxX + d, spos.y - d, spos.z], [spos.x + maxX + d, spos.y + ySize + d, spos.z ], [spos.x - d, spos.y +  ySize + d, spos.z]) true 
		d=5
		gw.setColor #fill white
		gw.setColor #line white

		gw.hPolyline #(spos - [d,d, 0], [spos.x + maxX + d, spos.y - d, spos.z], [spos.x + maxX + d, spos.y + ySize + d, spos.z ], [spos.x - d, spos.y +  ySize + d, spos.z]) true 
	
		for tl = 1 to LineCount do gw.hText (spos + [0, (ySize1 * (LineCount - tl)) - 3,0]) textArray[tl] color:white
		gw.enlargeUpdateRect #whole
		gw.updateScreen() 
	)
	
	function tagTimeCallback =
	(
		cTime = (currentTime as integer) / TicksPerFrame
		currentTags = #()
		
		for t=1 to tag_list.count do if cTime == tag_list[t][2] then	append currentTags tag_list[t][3]
		
		if currentTags.count > 0 then 
		(
			currentTags_Timer = #()
			for t=1 to currentTags.count do append currentTags_Timer (copy currentTags[t])
			ro_TagEditor.ti_TagTimer.ticks = 0
			ro_TagEditor.ti_TagTimer.active = true
			writeVPInformation currentTags black
		)
	)
	
	function readPresets confFile =
	(		
		ver = getIniSetting confFile "TimeTagger Presets" "Version"
		if ver != undefined do
		(
			ver = ver as integer
			if ver == 1 then
			(
				TE_TagPresets = #("None", "Selected Object Name")
				newTags = execute (getIniSetting confFile "TimeTagger Presets" "Names")
				join TE_TagPresets newTags
			)
		)
	)
		
	function savePresets confFile =
	(
		pList = #()
		for t=3 to TE_TagPresets.count do append pList TE_TagPresets[t]
		setIniSetting confFile "TimeTagger Presets" "Version" "1"
		setIniSetting confFile "TimeTagger Presets" "Names"  (with printAllElements on (pList as string))
	)

	rollout ro_TT_About "About TimeTagger"
	(
		label lbl_About_01 "TimeTagger 1.0" pos:[58,10]
		label lbl_About_02 "(c) 2005 by Andr Hotz" pos:[40,30]
		label lbl_About_022 "Piranha-Bytes, Pluto 13 GmbH" pos:[23,50]
		label lbl_About_03 "Use at your own risk!" pos:[45,80]
		label lbl_About_04 "Please support and buy our games ;)" pos:[8,100]
		hyperlink ui_AboutLabel3 "www.gothic2.com" address:"http://www.gothic2.com" pos:[50,120] 
		hyperlink ui_AboutLabel4 "andre@piranha-bytes.com" pos:[30,140] address:"mailto:andre@piranha-bytes.com"
	)


	rcMenu me_FileMenu
	(
		menuItem mi_SaveTags "Save Tags"
		menuItem mi_LoadTags "Load Tags"
		menuItem mi_LoadFromMax "Import from Max-File"
		separator sep1
		menuItem mi_Config "Config"
		menuItem mi_About "About"
		separator sep2
		menuItem mi_Refresh "Refresh"
		
		on mi_About picked do createDialog ro_TT_About width:190
		
		function saveTagFile saveName =
		(
			saveFile = openFile saveName mode:"w+"
			if saveFile != undefined do
			(
				print "TimeTag File" to:saveFile
				print tag_List.count to:saveFile
				for t=1 to tag_List.count do
				(
					print tag_List[t][2] to:saveFile
					print tag_List[t][3] to:saveFile
				)
				close saveFile
			)
		)
		
		function loadTagFile loadName clearTags =
		(
			oFile = openFile loadName mode:"r+"
			if oFile != undefined do
			(
				ident = readValue oFile 
				if ident == "TimeTag File" then
				(
					if clearTags == true then
					(
						FrameTagManager.ResetFrameTags()
						ro_TagEditor.updateTagList()
					)
					tagCount = readValue oFile 
					for t=1 to tagCount do
					(
						tagTime = readValue oFile 
						tagName = readValue oFile 
						ro_TagEditor.AddTTag tagName tagTime
					)
				)
				else messageBox "Incompatible File!!" title:"Error:"
				close oFile 
			)
		)
		
		on me_FileMenu open do
		(
			if tag_List.count == 0 then mi_SaveTags.enabled = false
			else mi_SaveTags.enabled = true
		)
		
		on mi_SaveTags picked do
		(
			saveName = getSaveFileName caption:"Export Tags to file..."
			if saveName != undefined do
			(
				filePath = getFileNamePath saveName
				saveFName = getFileNameFile saveName
				newSaveName = filePath + saveFName + ".tag"
				saveTagFile newSaveName
			)
		)
	
		on mi_LoadTags picked do
		(
			openName = getOpenFileName caption:"Import Tags from file..." types:"Tag File(*.tag)|*.tag|"
			if openName != undefined do
			(
				ask = queryBox "Delete existing tags?" title:"Load Time Tags:"
				loadTagFile openName ask
			)
		)
	
		on mi_LoadFromMax picked do
		(
			openName = getMAXOpenFileName() 
			if openName != undefined do
			(
				ask = queryBox "Delete existing tags?" title:"Load Time Tags:"
				holdMaxFile()
				loadMaxFile openName useFileUnits:false quiet:true
				ro_TagEditor.updateTagList()
				saveTagFile (TE_PATH + "_temp\\tmpTagList.tag")
				fetchMaxFile quiet:true
				ro_TagEditor.updateTagList()
				loadTagFile (TE_PATH + "_temp\\tmpTagList.tag") ask
				ro_TagEditor.updateTagList()
			)
		)
		on mi_Refresh picked do ro_TagEditor.updateTagList()
		on mi_Config picked do createDialog ro_TimeTaggerConfig	
	)

	rcMenu me_ListMenu
	(
		menuItem mi_Collapse "Collapse all"
		menuItem mi_Expand "Expand all"
		seperator sep1
		menuItem mi_SelectPos "Check all Position Channels"
		menuItem mi_SelectRot "Check all Rotation Channels"
		menuItem mi_SelectSca "Check all Scale Channels"
		
		on mi_Collapse picked do
		(
			for n=2 to ro_CreateFromKeys.ui_AX_ControllerList.nodes.count do
			(
				ro_CreateFromKeys.ui_AX_ControllerList.nodes[n].expanded = false
			)
		)
		on mi_Expand picked do
		(
			for n=2 to ro_CreateFromKeys.ui_AX_ControllerList.nodes.count do
			(
				ro_CreateFromKeys.ui_AX_ControllerList.nodes[n].expanded = true
			)
		)
	
		function checkByType chType =
		(
			counter = 1
			for o=1 to controllerList.count do
			(
				counter += 1
				for c=1 to controllerList[o].count do
				(
					counter += 1
					theNode = ro_CreateFromKeys.ui_AX_ControllerList.nodes[counter]
					isType = findString theNode.text chType
					if isType != undefined then theNode.checked = true
				)
			)
		)
	
		on mi_SelectPos picked do checkByType "Position"
		on mi_SelectRot picked do checkByType "Rotation"
		on mi_SelectSca picked do checkByType "Scale"
	)


	rollout ro_CreateFromKeys "Create Tags from Keys"
	(		
	 	activeXControl ui_AX_ControllerList "MSComctlLib.TreeCtrl" pos:[5,5] width:240 height:150
		button ui_CreateTags "Create Tags from selected Channels" pos:[5,160] width:240 height:20
		editText ui_TagPrefix "Tag Name: " pos:[5,185] fieldwidth:180 text:""
				
		function getObjControllers ctrl objNum =
		(
			for sa = 1 to ctrl.NumSubs do getObjControllers ctrl[sa] objNum

			if ((ctrl.keys != undefined) and (ctrl.keys.count > 0)) then
			(
				if ((ctrl.controller != undefined) and (ctrl.controller.supportsKeys == true)) then
				(
					append controllerList[objNum] ctrl
					append controllerNameList[objNum] (ctrl as string)
				)
			)
		)
	
		on ro_CreateFromKeys resized newSize do
		( 
			ui_AX_ControllerList.size = newSize - [10, 60]
			ui_CreateTags.pos.y = ui_AX_ControllerList.size.y + 10
			ui_TagPrefix.pos.y = ui_AX_ControllerList.size.y + 35
		)	
		
		function getSelChannel =
		(
			fo = 0
			fc = 0
			theNode = ui_AX_ControllerList.selectedItem
			
			if theNode.parent != undefined do
			(
				fo = findItem objectNameList theNode.parent.text
				if fo != 0 do
				(
					fc = findItem controllerNameList[fo] theNode.text
				)
			)
			return [fo,fc]
		)
		
		on ui_AX_ControllerList collapse theNode do if theNode.index == 1 then theNode.expanded = true
		on ui_AX_ControllerList nodeClick theNode do cc = getSelChannel()		
		on ui_AX_ControllerList nodeCheck theNode do
		(
			if theNode.index == 1 do for n=1 to ui_AX_ControllerList.nodes.count do ui_AX_ControllerList.nodes[n].checked = theNode.checked
			if theNode.index > 1 do
			(
				nodeText = theNode.text
				fo = findItem objectNameList nodeText
				if fo != 0 do
				(
					for n=1 to controllerList[fo].count do ui_AX_ControllerList.nodes[theNode.index + n].checked = theNode.checked
				)
			)
			ui_AX_ControllerList.refresh()
		)
		
		on ui_AX_ControllerList mouseUp mbutton shift xPos yPos do
		(
			if mButton == 2 do popupMenu me_ListMenu
		)
		
		function hasTag tagTime =
		(
			back = false
			ht = FrameTagManager.findItemByTime tagTime
			if ht != 0 then back = true
			return back
		)
		
		on ui_CreateTags pressed do
		(			
			tagName = ui_TagPrefix.text
			if tagName == "" then tagName = "NewTag"
			
			tagCount = 0
			
			liItems = ui_AX_ControllerList.nodes
			for n=1 to liItems.count do
			(
				if liItems[n].checked == true do
				(
					if liItems[n].parent != undefined do
					(
						fo = findItem objectNameList liItems[n].parent.text
						if fo != 0 do
						(
							fc = findItem controllerNameList[fo] liItems[n].text
							if fc != 0 do
							(
								theCtrl = controllerList[fo][fc]
								if theCtrl != undefined do
								(
									for k = 1 to theCtrl.keys.count do
									(
										keyTime = theCtrl.keys[k].time
										if (hasTag keyTime) == false then
										(
											ro_TagEditor.addTTag tagName keyTime
											tagCount = tagCount + 1
										)
									)
								)
							)
						)
					)
				)
			)
			messageBox ((tagCount as string) + " Tag(s) created!!") title:"Info:"
		)
		
		function readChannels =
		(
			objectList = getCurrentSelection()
			objectNameList = #()
			controllerList = #()
			controllerNameList = #()
			
			for o = 1 to objectList.count do  
			(
				append objectNameList objectList[o].name
				controllerList[o] = #()
				controllerNameList[o] = #()
				getObjControllers objectList[o].controller o
				getObjControllers objectList[o][4] o 
			)
			
			ui_AX_ControllerList.Nodes.clear()
			
			theRoot =  ui_AX_ControllerList.Nodes.Add()
			theRoot.text = "Animated Channels:"
			theRoot.expanded = true
			theRoot.foreColor = (color 0 0 200)
	
			for o = 1 to objectList.count do
			(
				newNode = ui_AX_ControllerList.nodes.add theRoot.index 4 "" objectList[o].name 0
				newNode.expanded = true
				newNode.foreColor = (color 0 100 0)
	
				for b = 1 to controllerList[o].count do
				(
					newSubNode = ui_AX_ControllerList.nodes.add newNode.index 4 "" (controllerList[o][b] as string) 0
					newSubNode.Expanded = true
					newSubNode.foreColor = (color 200 0 0)
				)
			)
		)
		
		on ui_GetObjs pressed do readChannels()
				
		on ro_CreateFromKeys open do
		(	
			controllerList = #()
			controllerNameList = #()
			objectList = #()
			objectNameList = #()
	
			ui_AX_ControllerList.Indentation = 28*20  --15
			ui_AX_ControllerList.LineStyle = #tvwTreeLines
			ui_AX_ControllerList.SingleSel = false
			ui_AX_ControllerList.HotTracking = true
			ui_AX_ControllerList.PathSeparator = ""
			ui_AX_ControllerList.checkboxes = true	
			ui_AX_ControllerList.LabelEdit = #tvwManual	
			ui_AX_ControllerList.HideSelection = false
			
			readChannels()
		)
		
		on ro_CreateFromKeys close do gc()		
	)

	rcmenu me_EditTools
	(
		menuItem mi_SetToCurrentFrame "Set Tag to current Frame"
		menuItem mi_CreateFromKeys "Create Tags from object keys"
		
		on mi_SetToCurrentFrame picked do
		(
			ro_TagEditor.ui_TagFrame.value = sliderTime
			ro_TagEditor.changeTag ro_TagEditor.ui_TagList.selection
		)
		
		on mi_CreateFromKeys picked do
		(
			os = getCurrentSelection()
			if os.count != 0 then
			(
				destroyDialog ro_CreateFromKeys
				createDialog ro_CreateFromKeys  height:210 width:250 style:#(#style_sysmenu, #style_titlebar, #style_border, #style_resizing, #style_minimizeBox)
			)
			else messageBox "No Object selected!!" title:"Warning:"
		)
	)

	rollout ro_TagEditor "pb  TimeTagger"
	(
		activeXControl ui_TagListAX "MSComctlLib.ListViewCtrl" pos:[5,5] width:165 height:140

		listbox ui_TagList pos:[5,5] width:165 height:10 items:#() visible:false
		spinner ui_TagFrame pos:[5,150] fieldwidth:35 range:[0,999999,0] scale:1 type:#integer
		editText ui_TagName pos:[55,150] fieldwidth:109
		label lbl_tp "Name Presets:" pos:[5,174]
		dropdownList ui_TagPreset pos:[80,171] width:90 height:20 items:TE_TagPresets
		
		button ui_AddTag "Add" pos:[5,200] width:45 height:20 tooltip:"Add new Time Tag"
		button ui_DelTag "Delete" pos:[50,200] width:50 height:20 tooltip:"Delete selected Time Tag"
		button ui_DelAll "Delete All" pos:[100,200] width:70 height:20 tooltip:"Delete all Time Tags"
		
		button ui_Edit "Editing Tools..." pos:[5,220] width:83 height:20
		button ui_File "File Tools.."  pos:[88,220] width:82 height:20
	
		timer ti_TagTimer interval:10 active:false
	
		on ti_TagTimer tick do
		(
			if tagDisplayTime != 0 then
			(
				if ti_TagTimer.ticks == tagDisplayTime do
				(
					ti_TagTimer.ticks = 0
					ti_TagTimer.active = false
				)
				if currentTags_Timer.count > 0 then writeVPInformation currentTags_Timer black
			)
			else ti_TagTimer.active = false
		)

		function setUI theTag =
		(
			if theTag != 0 then
			(
				ui_TagFrame.value = tag_List[theTag][2]
				ui_TagName.text = tag_List[theTag][3]
				fp = findItem ui_TagPreset.items ui_TagName.text
				if fp != 0 then ui_TagPreset.selection = fp
				else ui_TagPreset.selection = 1
			)
			else
			(
				ui_TagFrame.value = 0
				ui_TagName.text = ""
			)
		)
	
		function updateTagList =
		(
			tag_DisplayList = #()
			tag_List = #()
			ui_TagListAX.listItems.clear()
			tagCount = FrameTagManager.getTagCount()

			for t=1 to tagCount do
			(
				tagID = FrameTagManager.getTagID t
				tagTime = ((FrameTagManager.getTimeByID tagID) as integer) / TicksPerFrame
				tagName = FrameTagManager.getNameByID tagID
				
				append tag_List #(tagID, tagTime, tagName)
				append tag_DisplayList ((tagTime as string) + " " + tagName)
				
				newLI = ui_TagListAX.ListItems.Add()
				newLI.text = tagTime as string
				subLI = newLI.ListSubItems.Add()
				subLI.text = TagName
			)
			ui_TagList.items = tag_DisplayList
			
			if tagCount == 0 then
			(
				ui_TagList.enabled = false
				ui_TagFrame.enabled = false
				ui_TagName.enabled = false
				ui_TagPreset.enabled = false
				ui_DelTag.enabled = false
				ui_DelAll.enabled = false
				ui_TagFrame.value = 0
				ui_TagName.text = ""
			)
			else
			(
				ui_TagList.enabled = true
				ui_TagFrame.enabled = true
				ui_TagName.enabled = true
				ui_TagPreset.enabled = true
				ui_DelTag.enabled = true
				ui_DelAll.enabled = true
				ui_TagList.selection = 1
				ui_TagListAX.selectedItem = ui_TagListAX.ListItems[1]
				setUI 1
			)
		)
	
		function changeTag theTag =
		(
			if theTag != 0 do
			(
				NewTagTime = ui_TagFrame.value
				NewTagName = ui_TagName.text
				tagID = tag_List[theTag][1]
				FrameTagManager.setTimeByID tagID NewTagTime
				FrameTagManager.setNameByID tagID NewTagName
				newTagSelName = ((NewTagTime as string) + " " + NewTagName)
				updateTagList()
				newSel = findItem tag_DisplayList newTagSelName
				if newSel != 0 then
				(
					ui_TagList.selection = newSel
					ui_TagListAX.selectedItem = ui_TagListAX.ListItems[newSel]
					setUI newSel
				)
			)
		)

		on ui_TagList selected sel do setUI sel
		on ui_TagList doubleClicked sel do sliderTime = ui_TagFrame.value
		on ui_TagFrame changed val do changeTag ui_TagList.selection
		on ui_TagNAme entered newName do changeTag ui_TagList.selection

		on ui_TagPreset selected sel do
		(
			if sel == 2 then
			(
				curSel = (getCurrentSelection())[1]
				if curSel != undefined do
				(
					ui_TagName.text = curSel.name
					changeTag ui_TagList.selection
				)
				ui_TagPreset.selection = 1
			)
			if sel > 2 do
			(
				ui_TagName.text = ui_TagPreset.selected
				changeTag ui_TagList.selection
			)
		)

		function addTTag tagName tagTime =
		(
			FrameTagManager.createNewTag tagName tagTime scale:true
			newTagSelName = ((tagTime as string) + " " + tagName)
			updateTagList()
			newSel = findItem tag_DisplayList newTagSelName
			if newSel != 0 then
			(
				ui_TagList.selection = newSel
				ui_TagListAX.selectedItem = ui_TagListAX.ListItems[newSel]
				setUI newSel
			)
		)
		
		function DelTTag theTag =
		(
			if theTag != 0 do
			(
				tagID = tag_List[theTag][1]
				FrameTagManager.DeleteTag tagID
				updateTagList()
			)
		)
		
		on ui_AddTag pressed do
		(
			curFrame = (sliderTime as integer) / TicksPerFrame
			addTTag "NewTag" curFrame
		)
		
		on ui_Edit pressed do popupMenu me_EditTools 
		on ui_DelTag pressed do	DelTTag ui_TagList.selection
		on ui_DelAll pressed do
		(
			ask = queryBox "Really delete ALL Timetags?" title:"Warning:"
			if ask == true then 
			(
				FrameTagManager.ResetFrameTags()
				updateTagList()
			)
		)
	
		on ui_File pressed do popupMenu me_FileMenu
		
		on ui_TagListAX ItemClick theItem do 
		(
			idx = theItem.index		
			ui_TagList.selection = idx
			setUI idx
		)
		
		on ui_TagListAX dblCLick do	sliderTime = ui_TagFrame.value
		
		on ro_TagEditor resized newSize do
		(
			if newSize != [0,0] do
			(
				oldSize = ui_TaglistAX.size.y
				ui_TaglistAX.size = [newSize.x - 10, newSize.y - 105]
				ui_TagListAX.columnHeaders[2].width = (newSize.x - 50) * 22		
				uiCtrls = ro_TagEditor.controls
				for c=2 to uiCtrls.count do uiCtrls[c].pos.y = ui_TaglistAX.size.y + (uiCtrls[c].pos.y - oldSize)
			)
		)
		
		on ro_TagEditor open do
		(
			sDir = getDir #scripts
			makeDir (sDir    + "\\PB_Data")
			makeDir (sDir    + "\\PB_Data\\TimeTagger")
			TE_Path = (sDir  + "\\PB_Data\\TimeTagger\\")
			makeDir (TE_Path + "Config")
			makeDir (TE_Path + "_temp")
			
			readPresets (TE_Path + "Config\\Presets.pre")
			ui_TagPreset.items = TE_TagPresets

			ui_tagListAX.MousePointer = #ccArrow
			ui_tagListAX.GridLines = false
			ui_tagListAX.View = #lvwReport 
			ui_tagListAX.hideSelection = false
			ui_tagListAX.LabelEdit = #lvwManual
			ui_tagListAX.FullRowSelect = true
			ui_tagListAX.HotTracking = true
			ui_tagListAX.HideColumnHeaders = false
			ui_tagListAX.AllowColumnReorder = false
			ui_tagListAX.backcolor = (color 100 75 65)
			ui_tagListAX.forecolor = (color 200 200 200)
			
			cols = ui_tagListAX.columnHeaders
			lNameC = cols.Add()
			lNameC.text = "Time"
			lNameC.width = 1000
			pMixC = cols.Add()
			pMixC.text = "Name"
			pMixC.width = 2750

			updateTagList()
			
			--unregisterTimeCallBack tagTimeCallback
			registerTimeCallBack tagTimeCallback
		)
		
		on ro_TagEditor close do 
		(
			destroyDialog ro_TimetaggerConfig
			destroyDialog ro_CreateFromKeys
			destroyDialog ro_TT_About
			unregisterTimeCallBack tagTimeCallback
			gc()
		)
	)
	
	rollout ro_TimeTaggerConfig "TimeTagger Config:"
	(
		comboBox ui_PresetList "Tag Name Presets:" pos:[5,5] width:150 height:12 items:#()
		button ui_AddPreset "Add" pos:[5,200] width:40 height:20 tooltip:"Add name preset"
		button ui_DelPreset "Del" pos:[45,200] width:40 height:20 tooltip:"Delete selected name preset"
		button ui_LoadPresets "Load" pos:[85,200] width:35 height:20 tooltip:"Load name presets from file"
		button ui_SavePresets "Save" pos:[120,200] width:35 height:20 tooltip:"Save name presets to file"
		spinner ui_tagDisplayTime "VP Display Time (ms): " pos:[5,225] range:[0,9999999, tagDisplayTime] fieldwidth:35 scale:1 type:#integer
		
		on ui_tagDisplayTime changed val do tagDisplayTime = val
		
		function updatePresetList =
		(
			presetDisplayList = #()
			for p=3 to TE_TagPresets.count do append presetDisplayList TE_TagPresets[p]
			ui_PresetList.items = presetDisplayList
			ro_TagEditor.ui_TagPreset.items = TE_TagPresets
		)
		
		on ui_LoadPresets pressed do
		(
			confName = getOpenFilename caption:"Load Name Presets:" filename:(TE_Path + "Config\\Presets.pre") types:"Presets(*.pre)|*.pre|"
			if confName != undefined do readPresets confName
			updatePresetList()
		)
		on ui_SavePresets pressed do
		(
			confName = getSaveFilename caption:"Save Name Presets:" filename:(TE_Path + "Config\\Presets.pre") types:"Presets(*.pre)|*.pre|"
			if confName != undefined do savePresets confName
			updatePresetList()
		)
		
		on ui_PresetList entered newName do
		(
			selP = ui_PresetList.selection + 2
			TE_TagPresets[selP] = newName
			updatePresetList()
		)
		
		on ui_AddPreset pressed do
		(
			append TE_TagPresets "NewPreset"
			updatePresetList()			
		)
		
		on ui_DelPreset pressed do
		(
			selP = ui_PresetList.selection + 2
			if selP > 2 then
			(
				deleteItem TE_TagPresets selP
				updatePresetList()			
			)
		)
		
		on ro_TimeTaggerConfig open do updatePresetList()
	)
	createDialog ro_TagEditor pos:[100,100] height:245 width:175 style:#(#style_sysmenu, #style_titlebar, #style_border, #style_resizing, #style_minimizeBox)
)
